package BSS.Devices;

import BSS.*;

import java.util.Map;
import java.util.TreeMap;

public abstract class BaseDevice implements Device, Host, EnvironmentCodes
{
    public final static Map DEVICE_NAMES = new TreeMap();
    static
    {
        DEVICE_NAMES.put(new Integer(DEV_CONTACT_SENSOR), "ContactSensor");
        DEVICE_NAMES.put(new Integer(DEV_SMARTCARD_DRIVE), "SmartCardDrive");
        DEVICE_NAMES.put(new Integer(DEV_LED), "LED");
        DEVICE_NAMES.put(new Integer(DEV_LOCK), "Lock");
        DEVICE_NAMES.put(new Integer(DEV_PHOTO_SENSOR), "PhotoSensor");
        DEVICE_NAMES.put(new Integer(DEV_MOTOR), "Motor");
        DEVICE_NAMES.put(new Integer(DEV_ENERGIZER), "Energizer");
        DEVICE_NAMES.put(new Integer(DEV_METAL_SENSOR), "MetalSensor");
        DEVICE_NAMES.put(new Integer(DEV_KEYBOARD), "Keyboard");
        DEVICE_NAMES.put(new Integer(DEV_PRINTER), "Printer");
        DEVICE_NAMES.put(new Integer(DEV_FIRE_SENSOR), "FireSensor");
        DEVICE_NAMES.put(new Integer(DEV_SMUDGE), "Smudge");
        DEVICE_NAMES.put(new Integer(DEV_CAMERA), "Camera");
        DEVICE_NAMES.put(new Integer(DEV_INTERFERENCE), "Interference");
        DEVICE_NAMES.put(new Integer(DEV_FIRE_EXTINGUISHER), "FireExtinguisher");
        DEVICE_NAMES.put(new Integer(DEV_LIFT_CONTROLLER), "LiftController");
        DEVICE_NAMES.put(new Integer(DEV_VOLUME_SENSOR), "VolumeSensor");
    }

    int          _id;
    Logger       _logger;
    Map          _config;
    Communicator _comm;
    Controller   _con;

    public BaseDevice(int id, Logger logger, Map configuration, Communicator communicator)
    {
        if (null == logger)
        {
            throw new IllegalArgumentException("Null logger.");
        }
        if (null == configuration)
        {
            throw new IllegalArgumentException("Null configuration.");
        }
        if (null == communicator)
        {
            throw new IllegalArgumentException("Null communicator.");
        }
        _id = id;
        _logger = logger;
        _config = configuration;
        _comm = communicator;
    }

    public String toString()
    {
        return "Controller#" + _id;
    }

    public void setController(Controller controller)
    {
        _con = controller;
    }

    protected int ID()
    {
        return _id;
    }

    //--- Device interface methods ---
    public byte getStatus(int device_type, int device_id)
    {
        byte value = 0;
        log_get_status(false, device_type, device_id, value);
        return value;
    }

    public byte[] readData(int device_type, int device_id)
    {
        byte[] value = new byte[0];
        log_read_data(false, device_type, device_id, value);
        return value;
    }

    public void setStatus(int device_type, int device_id, byte status)
    {
        log_set_status(false, device_type, device_id, status);
    }

    public void writeData(int device_type, int device_id, byte[] data)
    {
        log_write_data(false, device_type, device_id, data);
    }

    public Map loadConfiguration()
    {
        log(3, toString() + " loads configuration.");
        return _config;
    }

    public Message sendMessage(Message msg)
    {
        return _comm.sendMessage(msg);
    }

    public void sendSMS(SMS sms)
    {
        _comm.sendSMS(sms);
    }

    //--- Host interface methods ---
    public void updateStatus(int device_type, int device_id, byte value)
    {
        log_update_status(false, device_type, device_id, value);
    }

    public void updateData(int device_type, int device_id, byte value[])
    {
        log_update_data(false, device_type, device_id, value);
    }

    public void interrupt(int event)
    {
        _con.envOnEvent(event);
    }

    public Message receiveMessage(Message msg)
    {
        return _con.msgOnReceive(msg);
    }

    public void receiveSMS(SMS sms)
    {
        _con.smsOnReceive(sms);
    }

    //--- Logging methods ---
    protected void log(int level, String message)
    {
        _logger.log(level, message);
    }

    protected void log_get_status(boolean read, int device_type, int device_id, byte value)
    {
        if (read)
        {
            log(3, toString() + " got status `" + value + "` of device " + devTypeToString(device_type) + ":" + device_id);
        }
        else
        {
            log(1, toString() + " attempted to get status of device " + devTypeToString(device_type) + ":" + device_id + " at " + toString());
        }
    }

    protected void log_read_data(boolean read, int device_type, int device_id, byte[] value)
    {
        if (read)
        {
            log(3, toString() + " read data buffer `" + value + "` of device " + devTypeToString(device_type) + ":" + device_id);
        }
        else
        {
            log(1, toString() + " attempted to read data buffer of device " + devTypeToString(device_type) + ":" + device_id + " at " + toString());
        }
    }

    protected void log_set_status(boolean wrote, int device_type, int device_id, byte value)
    {
        if (wrote)
        {
            log(3, toString() + " set status `" + value + "` of device " + devTypeToString(device_type) + ":" + device_id);
        }
        else
        {
            log(1, toString() + " attempted to set status of device " + devTypeToString(device_type) + ":" + device_id + " at " + toString());
        }
    }

    protected void log_write_data(boolean wrote, int device_type, int device_id, byte[] value)
    {
        if (wrote)
        {
            log(3, toString() + " wrote `" + value + "` of device " + devTypeToString(device_type) + ":" + device_id);
        }
        else
        {
            log(1, toString() + " attempted to write to data buffer of device " + devTypeToString(device_type) + ":" + device_id + " at " + toString());
        }
    }

    protected void log_update_status(boolean updated, int device_type, int device_id, byte value)
    {
        if (updated)
        {
            log(3, "Status of device " + devTypeToString(device_type) + ":" + device_id + " updated to `" + value + "` at " + toString());
        }
        else
        {
            log(1, "Attempt to update status of device " + devTypeToString(device_type) + ":" + device_id + " at " + toString());
        }
    }

    protected void log_update_data(boolean updated, int device_type, int device_id, byte[] value)
    {
        if (updated)
        {
            log(3, "Data buffer of device " + devTypeToString(device_type) + ":" + device_id + " updated to `" + value + "` at " + toString());
        }
        else
        {
            log(1, "Attempt to update data buffer of device " + devTypeToString(device_type) + ":" + device_id + " at " + toString());
        }
    }

    private String devTypeToString(int device_type)
    {
        String desc = (String)DEVICE_NAMES.get(new Integer(device_type));
        return (null == desc) ? String.valueOf(device_type) : desc;
    }
}
